@@ -37,7 +37,7 @@ class http_client(object):
         self.url = self.credentials['url']
 
         # FIXME change to alive endpoint
-        self.check_login_worked()
+        self._check_login_worked()
         self.version_check()
         
         # see if there there is another server url that the web client has decided would be better to use
@@ -58,7 +58,7 @@ class http_client(object):
             
         return
 
-    def check_login_worked(self):
+    def _check_login_worked(self):
         #first, we check for invalid grant errors
         try:
             r=self.client.get(self.url+"/users/whoami") #random method to call web api, so that we can check the response code
@@ -66,19 +66,13 @@ class http_client(object):
                 logger.error(f"Call to whoami failed with HTTP Code: {r.status_code}. Attempting to create new token")
                 self.client = login.create_token()
             else:
-                logger.info("Login Succesfull")                
+                logger.info("Login Succesfull")
 
         except InvalidGrantError:
             logger.error("Invalid Grant Exception Thrown. Attempting to create new token")
             self.client = login.create_token()
         return 
 
-    def get_custom_endpoint(self, endpoint_url):
-        """Call a custom endpoint url.
-        Current endpoint urls can be found at live.openlab.app/swagger"""
-        r = self.client.get(self.url + endpoint_url)
-        return r
-
     def labels(self):
         r=self.client.get(self.url+"/labels/")
         return self.standard_response(r, 200) 
@@ -136,28 +130,38 @@ class http_client(object):
 
     def simulations(self):
         """
-        Returns all simulations of the current user
+        Returns a list all simulations of the current user
+        Each entry being a dictionary with some simulation information
         """        
         r=self.client.get(self.url+"/simulations")
         return self.standard_response(r, 200)
 
     def check_if_simulation_exists(self,sim_id):
+        """
+        Deprecated: Use _check_if_simulation_exists() instead
+        """
+        logger.warning("check_if_simulation_exists is Deprected. Use _check_if_simulation_exists instead")
+        return self._check_if_simulation_exists(sim_id)
+
+    def _check_if_simulation_exists(self,sim_id):
         simulations = self.simulations()
         sim_exists = False
         for simulation in simulations:
             if simulation['SimulationID'] == sim_id:
                 sim_exists = True
-        if not sim_exists:
-            raise exceptions.SimulationNotFound("Simulation {} does not exist".format(sim_id))
+        return sim_exists            
 
     def set_setpoints(self, sim_id, step, setpoints, shouldComplete=False):
         """
-        Sends setpoints to a given simulation
+        Sends manually formatted setpoints to a given simulation outside of Simulation class (i.e. steps a simulation forward)
+        Setpoints must be a single dictionary, and formatted as: {'tag':value}
+        For a list of openlab setpoint tags, see openlab.setpoints.valid_tags. Tags are string sensitive
+        Passing in True to shouldComplete, will complete the simulation
         """
         data= self.format_set_point_data(step, setpoints, shouldComplete)
         r=self.client.post(self.url+"/simulations/"+sim_id+"/setpoints", json= data)
-        if r.status_code == 404:
-            self.check_if_simulation_exists(sim_id)
+        if r.status_code == 404 and self.check_if_simulation_exists(sim_id) is False:
+            raise exceptions.SimulationNotFound("Simulation {} does not exist".format(sim_id))
 
         return self.standard_response(r, 200)
     
@@ -168,9 +172,9 @@ class http_client(object):
         #timestep = self.last_timestep(sim_id)
         r=self.client.get(self.url+"/simulations/"+ sim_id+ "/setpoints")
         
-        if r.status_code == 404:
-            self.check_if_simulation_exists(sim_id)
-        
+        if r.status_code == 404 and self.check_if_simulation_exists(sim_id) is False:
+            raise exceptions.SimulationNotFound("Simulation {} does not exist".format(sim_id))
+            
         return self.standard_response(r, 200)
 
     def last_setpoint(self, sim_id):
@@ -180,8 +184,8 @@ class http_client(object):
         #timestep = self.last_timestep(sim_id)
         r=self.client.get(self.url+"/simulations/"+ sim_id+ "/setpoints/last")
 
-        if r.status_code == 404:
-            self.check_if_simulation_exists(sim_id)
+        if r.status_code == 404 and self.check_if_simulation_exists(sim_id) is False:
+            raise exceptions.SimulationNotFound("Simulation {} does not exist".format(sim_id))
         
         return self.standard_response(r, 200)
 
@@ -191,16 +195,16 @@ class http_client(object):
         """
         r=self.client.get(self.url+"/simulations/"+ sim_id+ "/timestep")
 
-        if r.status_code == 404:
-            self.check_if_simulation_exists(sim_id)
+        if r.status_code == 404 and self.check_if_simulation_exists(sim_id) is False:
+            raise exceptions.SimulationNotFound("Simulation {} does not exist".format(sim_id))            
 
         return self.standard_response(r, 200)
     
     def get_simulation_status(self,sim_id):
         r = self.client.get(self.url+"/simulations/"+sim_id+"/status")
 
-        if r.status_code == 404:
-            self.check_if_simulation_exists(sim_id)
+        if r.status_code == 404 and self.check_if_simulation_exists(sim_id) is False:
+            raise exceptions.SimulationNotFound("Simulation {} does not exist".format(sim_id))            
 
         return self.standard_response(r, 200)
 
@@ -256,7 +260,39 @@ class http_client(object):
         r=self.client.put(self.url+"/configurations/"+config_id, json=body)
         return self.standard_response(r, 200)
     
+    def _check_if_configuration_belongs_to_user(self, config_id):
+        """
+        Takes a config guid string. 
+        Returns false if configuration doesn't exist or if it exists but isn't found in user configurations
+        """
+        try:    
+            info = self.configuration_info(config_id)
+        except exceptions.ConfigurationNotFound:
+            return False
+
+        if info is None:
+            # Raise ConfigurationNotFound here instead??
+            return False
+
+        user_configurations = self.configurations()
+        config_owned_by_user = False
+        for configuration in user_configurations:
+            if configuration['ConfigurationID'] == config_id:
+                config_owned_by_user = True
+        
+        return config_owned_by_user
+
     def check_if_configuration_id_exists(self,config_id):
+        """
+        Deprecated: Use _check_if_configuration_id_exists() instead
+        """
+        logger.warning("check_if_configuration_id_exists is Deprected. Use _check_if_configuration_id_exists instead")
+        return self._check_if_configuration_id_exists(config_id)
+
+    def _check_if_configuration_id_exists(self,config_id):
+        """
+        Not necessarily owned by the user
+        """
         configurations = self.configurations()
         config_exists = False
         for configuration in configurations:
@@ -266,6 +302,18 @@ class http_client(object):
             return False
 
     def check_if_configuration_name_exists(self,config_name):
+        """
+        Deprecated: Use _check_if_configuration_name_exists() instead
+        """
+        logger.warning("check_if_configuration_name_exists is Deprected. Use _check_if_configuration_name_exists instead")
+        return self._check_if_configuration_name_exists(config_name)
+
+    def _check_if_configuration_name_exists(self,config_name):
+        """
+        Takes a configuration name string (case sensitive), and
+        Returns True/False depending on 
+
+        """
         configurations = self.configurations()
         config_exists = False
         for configuration in configurations:
@@ -292,7 +340,7 @@ class http_client(object):
         r=self.client.get(self.url+"/configurations/"+ name)
 
         if r.status_code == 404:
-            exists = self.check_if_configuration_name_exists(name)
+            exists = self._check_if_configuration_name_exists(name)
             if exists is False:
                 raise exceptions.ConfigurationNotFound(f"{r.text}\nConfiguration \"{name}\" does not exist")
 
@@ -305,7 +353,8 @@ class http_client(object):
 
     def configuration_simulations(self, config_id):
         """
-        Returns all the simulations for a given configuration ID
+        Returns a list of all the simulations for a given configuration
+        With each entry of said list, being a dictionary of simulation info
         """
         r = self.client.get(self.url+"/configurations/"+str(config_id)+"/simulations")
 
@@ -316,44 +365,71 @@ class http_client(object):
 
     def delete_simulation(self, simulationID):
         """
-        Deletes a given simulation
-        Prompt will be given to confirm
+        Deletes a given simulation.
+        Takes a simulation id
+        Returns True when HTTP code 204 is returned indicating succesful deletion
+        Raises SimulationNotFound when HTTP code 404 is returned
+        Else returns False with error log
         """
-        answer = input("Are you sure you want to delete Simulation {}\t(y/n)".format(simulationID))
-        if answer == "y":
-            self.client.delete(self.url+"/simulations/"+simulationID)
+        r = self.client.delete(self.url+"/simulations/"+simulationID)
+
+        if r.status_code == 204:
             logger.info(f"Simulation {simulationID} was deleted")
-        else:
-            return
+            return True
+        elif r.status_code == 404:
+            raise exceptions.SimulationNotFound(f"Simulation {simulationID} does not exist")
+        else:            
+            logger.error(f"Unable to delete simulation {simulationID}")
+            return False
 
     def delete_configuration(self, configurationID: str):
         """
-        Deletes a given configuration
+        Deletes a given configuration and all it's simulations
+        Takes a configuration id
+        Returns True when HTTP code 204 is returned indicating succesful deletion, 
+        Raises ConfigurationNotFound exception when HTTP code 404 is returned
+        Else returns False with error log
         """
-        self.client.delete(self.url+"/configurations/"+configurationID)
-
+        r = self.client.delete(self.url+"/configurations/"+configurationID)
+        print(r)
+        if r.status_code == 204:
+            logger.info(f"Configuration {configurationID} and all its simulations were deleted")
+            return True
+        elif r.status_code == 404:
+            raise exceptions.ConfigurationNotFound(f"Configuration {configurationID} does not exist")
+        else:
+            logger.error(f"Unable to delete configuration {configurationID}")
+            return False
 
     def simulation_by_id(self, sim_id):
         """
-        Returns simulation with a given id
+        Takes the simulation id of a simulation that has already been created
+        Returns an openlab.Simulation instance
         **Note you must use the id, not the name, as multiple simulations can have the same name
+        Simulation ID can be found in the url when viewing a simulation in live.openlab.app
         """
         if type(sim_id) is not str:
             raise TypeError("Sim-id must be a string")        
 
         r=self.client.get(self.url+"/simulations/"+ sim_id)
 
-        if r.status_code == 404:
-            self.check_if_simulation_exists(sim_id)
-            
-        return self.standard_response(r, 200)
+        if r.status_code == 404 and self.check_if_simulation_exists(sim_id) is False:
+            raise exceptions.SimulationNotFound("Simulation {} does not exist".format(sim_id))            
+
+        res = self.standard_response(r, 200)
+        config_id = res['ConfigurationID']
+        
+        sim = Simulation(config_id, sim_id, self)
+       
+        return sim
     
     def create_simulation(self, config_name, sim_name, initial_bit_depth, **kwargs):
         """
-        Creates a simulation
-        **kwargs influx_type and TimeStep
-        Run by calling step() method
-        Returns a Simulation Class
+        Creates a simulation and returns an openlab.Simulation class instance
+        Takes
+        **kwargs are influx_type and TimeStep (1 for steady-state, 0.1 for transient)
+        influx_types should be a dictionary, and examples for the three types (direct influx/direct loss/based on geopressure)
+        can be found 
         """
         kwargs.setdefault('influx_type', {})
         kwargs.setdefault('TimeStep', 1)
@@ -419,8 +495,8 @@ class http_client(object):
     def simulation_timestep(self, sim_id):
         r = self.client.get(self.url+"/simulations/"+sim_id+"/timestep")
         
-        if r.status_code == 404:
-            self.check_if_simulation_exists(sim_id)
+        if r.status_code == 404 and self.check_if_simulation_exists(sim_id) is False:
+            raise exceptions.SimulationNotFound("Simulation {} does not exist".format(sim_id))
 
         return self.standard_response(r, 200)
 
@@ -438,13 +514,10 @@ class http_client(object):
         if timeStep is None:
             timeStep = self.simulation_timestep(sim_id) + 1
             setpoints['Timestep'] = timeStep
-        elif timeStep > 0:
-            setpoints['Timestep'] = timeStep
-            setpoints['Data'] = {} # blank setpoints
-            setpoints['ShouldComplete'] = True
         else:
-            setpoints['Timestep'] = 0
+            setpoints['Timestep'] = timeStep
 
+        setpoints['Data'] = {} # blank setpoints
         setpoints['ShouldComplete'] = True
 
         body.append(setpoints)
@@ -598,7 +671,14 @@ class Simulation(http_client,metaclass=IterSimulation):
         self._registry.append(self) #add the simulation to the iterable registry 
         self.connecting_previously = False
         self.currently_connecting = False
-        self.timeStep = 0
+        # Make none so we can print out stepping first step in step 
+        self.timeStep = None #client.last_timestep(sim_id) or 0
+
+        status = self.get_status()
+        if status == "Created" or status == "Initialized" or status == "Running":
+            self.is_running = True
+        else:
+            self.is_running = False
     
     def __iter__(self):
         return iter(self)
@@ -680,7 +760,10 @@ class Simulation(http_client,metaclass=IterSimulation):
     def step(self,timeStep):
         """
         Steps the simulation one step forward
-        """ 
+        """
+        if self.timeStep is None:
+            self.timeStep = self.http_client.last_timestep(self.sim_id)
+            self.logger.info(f"Stepping first step ({self.timeStep})")
         # verify the time step is correct
         if timeStep != self.timeStep + 1:
             raise Exception(f"Trying to set a set a setpoint ({timeStep}) not at the immediate next timestep ({self.timeStep+1}).")
@@ -716,9 +799,8 @@ class Simulation(http_client,metaclass=IterSimulation):
             if "Connection" not in tags:
                 tags.append("Connection")
         
-
             attempts = 0
-            #loop until we get data or reach the max amount of attempts            
+            #loop until we get data or reach the max amount of attempts
             while(True):
                 #get the http_client to request simulation results
                 res = http_client.get_simulation_results(self.http_client, self.sim_id,timeStep, timeStep, self.filter_depth_based_results, tags)
@@ -728,14 +810,21 @@ class Simulation(http_client,metaclass=IterSimulation):
                     break
 
                 elif attempts >= self.max_results_attempts:
+                    status = self.get_status()                    
+                    if self.is_running is True:
+                        if status ==  "Completed":
+                            raise exceptions.SimulationCompleted(f"Sim {self.sim_id} was unexpectedly completed")
+                        if status == "Aborted":
+                            raise exceptions.SimulationAborted(f"Sim {self.sim_id} was unexpectedly aborted")
                     # Increase wait time. Something is taking a long time
                     time.sleep(1)
                 
                 else:
                     attempts = attempts + 1
-                    
-                    if attempts == self.max_results_attempts:
-                        self.logger.warning(f"Something is taking a long time at timestep {timeStep}")
+                    if attempts == self.max_results_attempts:                        
+                        if self.timeStep > 1:
+                            # First step always takes some time probably because initializing
+                            self.logger.warning(f"Something is taking a long time at timestep {timeStep}")
 
                     time.sleep(0.02)
             
