@@ -23,6 +23,24 @@ class SimulationNotFound(Exception):
     """
     Raise when simulation is not found in OpenLab
     """
+    def __init__(self, message):
+        # Call the base class constructor with the parameters it needs
+        super().__init__(message)
+        self.message = message
+
+class SimulationCompleted(Exception):
+    """
+    Raised when a running simulation is unexpectedly completed
+    """
+    def __init__(self, message):
+        # Call the base class constructor with the parameters it needs
+        super().__init__(message)
+        self.message = message
+
+class SimulationAborted(Exception):
+    """
+    Raised when a running simulation is unexpectedly aborted
+    """
     def __init__(self, message):
         # Call the base class constructor with the parameters it needs
         super().__init__(message)
